@@ -1,4 +1,4 @@
-from typing import Sequence, List, Union, Optional, Tuple
+from typing import Sequence, List, Union, Optional
 
 import numpy as np
 import pandas as pd
@@ -17,12 +17,15 @@ class ClassificationDataset(NLPDataset):
         df: Optional[pd.DataFrame] = None,
         csv_file: Optional[str] = None,
         in_memory: bool = True,
-        no_label: bool = False,
         is_multilabel: bool = True,
         max_length: Optional[int] = None,
         text_segmenter: str = "char",
         text_dtype: tf.DType = tf.int32,
         label_dtype: tf.DType = tf.float32,
+        text_padding_shape: tuple = (None,),
+        label_padding_shape: tuple = (None,),
+        text_padding_value: Union[str, int, float, None] = None,
+        label_padding_value: Union[str, int, float, None] = None,
     ):
         self.vocab = vocab
         self.labels = list(labels)
@@ -32,30 +35,16 @@ class ClassificationDataset(NLPDataset):
             df=df,
             csv_file=csv_file,
             in_memory=in_memory,
-            no_label=no_label,
             text_segmenter=text_segmenter,
             max_length=max_length,
-            na_value="" if is_multilabel else "NULL",
-            column_dtypes=["str", "str"],
             text_dtype=text_dtype,
             label_dtype=label_dtype,
+            text_padding_shape=text_padding_shape,
+            label_padding_shape=label_padding_shape,
+            text_padding_value=text_padding_value or vocab[vocab.pad],
+            label_padding_value=label_padding_value or 0.0,
         )
 
-    @property
-    def y(self) -> Union[List[str], List[List[str]]]:
-        if self.no_label:
-            return []
-        return [
-            data[-1].decode("utf-8").split("|")
-            if self.is_multilabel
-            else data[-1].decode("Utf-8")
-            for data in self._original_dataset.as_numpy_iterator()
-        ]
-
-    @property
-    def batch_padding_shapes(self) -> List[Tuple]:
-        return ((None,), (None,))[: -1 if self.no_label else None]
-
     def _text_transform(self, text: tf.Tensor) -> np.ndarray:
         tokens = super()._text_transform(text)
         return np.array(
@@ -69,18 +58,18 @@ class ClassificationDataset(NLPDataset):
         return res
 
     def _label_transform(self, label: tf.Tensor) -> np.ndarray:
-        _label = super()._label_transform(label)
+        label = super()._label_transform(label)
         if self.is_multilabel:
-            labels = _label.split("|")
+            labels = label.split("|")
         else:
-            labels = [_label]
+            labels = [label]
         return self._label_binarizer(labels)
 
     def to_tfrecord(self, filename: str) -> None:
         def func(text: np.ndarray, label: np.ndarray):
             return tf.reshape(
                 serialize_example(
-                    (self._text_transform(text), self._label_transform(label)),
+                    (self.text_transform(text), self.label_transform(label)),
                     ("tensor", "tensor"),
                 ),
                 (),
