@@ -7,5 +7,7 @@ def get_segmenter(name):
         return functools.partial(jieba.lcut, HMM=False)
     elif name == 'list' or name == 'char':
         return list
+    elif name is None:
+        return lambda x: x
     else:
         raise ValueError('unknown segmenter %s' % name)
