@@ -12,7 +12,7 @@ from sknlp.layers import (
 )
 from sknlp.vocab import Vocab
 from sknlp.data import BertClassificationDataset
-from sknlp.module.text2vec import Bert2vec
+from ..text2vec import Bert2vec
 from .deep_classifier import DeepClassifier
 
 
@@ -23,13 +23,9 @@ class BertClassifier(DeepClassifier):
         is_multilabel: bool = True,
         segmenter: Optional[str] = None,
         embedding_size: int = 100,
-        use_batch_normalization: bool = True,
         max_sequence_length: int = 120,
         num_fc_layers: int = 2,
         fc_hidden_size: int = 256,
-        fc_activation: str = "tanh",
-        fc_momentum: float = 0.9,
-        fc_epsilon: float = 1e-5,
         output_dropout: float = 0.5,
         text2vec: Optional[Bert2vec] = None,
         **kwargs
@@ -38,21 +34,24 @@ class BertClassifier(DeepClassifier):
             classes,
             is_multilabel=is_multilabel,
             segmenter=segmenter,
-            algorithm="bert",
+            algorithm="bert_classifier",
             embedding_size=embedding_size,
-            use_batch_normalization=use_batch_normalization,
             max_sequence_length=max_sequence_length,
             text2vec=text2vec,
             **kwargs
         )
+        # if self._text2vec is not None:
+        #     self.preprocessing_layer = BertPreprocessingLayer(
+        #         self._text2vec.vocab.sorted_tokens
+        #     )
+        
         self.num_fc_layers = num_fc_layers
         self.fc_hidden_size = fc_hidden_size
-        self.fc_activation = fc_activation
-        self.fc_momentum = fc_momentum
-        self.fc_epsilon = fc_epsilon
-        self.output_dropout = output_dropout
         self.inputs = tf.keras.Input(shape=(), dtype=tf.string, name="text_input")
 
+        # self.inputs = tf.keras.Input(shape=(), dtype=tf.string, name="text_input")
+        self._output_dropout = output_dropout
+
     def create_dataset_from_df(
         self, df: pd.DataFrame, vocab: Vocab, segmenter: str, labels: Sequence[str]
     ) -> BertClassificationDataset:
@@ -60,21 +59,29 @@ class BertClassifier(DeepClassifier):
             vocab,
             list(labels),
             df=df,
-            is_multilabel=self.is_multilabel,
-            max_length=self.max_sequence_length,
+            is_multilabel=self._is_multilabel,
+            max_length=self._max_sequence_length
         )
 
+    def get_inputs(self) -> tf.Tensor:
+        return self.inputs
+
+    def get_outputs(self) -> tf.Tensor:
+        return self.build_output_layer(self.build_encode_layer(self.get_inputs()))
+
     def build_encode_layer(self, inputs: tf.Tensor) -> tf.Tensor:
-        preprocessing_layer = BertPreprocessingLayer(self.text2vec.vocab.sorted_tokens)
+        preprocessing_layer = BertPreprocessingLayer(
+            self._text2vec.vocab.sorted_tokens
+        )
         token_ids = preprocessing_layer(inputs)
-        _, _, cls = self.text2vec(
+        _, _, cls = self._text2vec(
             [
                 token_ids,
                 K.zeros_like(token_ids, dtype=tf.int64),
             ]
         )
-        if self.output_dropout:
-            return Dropout(self.output_dropout, name="embedding_dropout")(cls)
+        if self._output_dropout:
+            return Dropout(self._output_dropout)(cls)
         return cls
 
     def build_output_layer(self, inputs: tf.Tensor) -> tf.Tensor:
@@ -82,15 +89,11 @@ class BertClassifier(DeepClassifier):
             self.num_fc_layers,
             hidden_size=self.fc_hidden_size,
             output_size=self.num_classes,
-            activation=self.fc_activation,
-            batch_normalization=self.use_batch_normalization,
-            momentum=self.fc_momentum,
-            epsilon=self.fc_epsilon,
             name="mlp",
         )(inputs)
 
     def get_config(self) -> Dict[str, Any]:
-        return {**super().get_config(), "output_dropout": self.output_dropout}
+        return {**super().get_config(), "output_dropout": self._output_dropout}
 
     def get_custom_objects(self) -> Dict[str, Any]:
         return {
