@@ -1,11 +1,11 @@
-from typing import Sequence, List, Dict, Any, Optional, Union, Tuple
+from typing import Sequence, List, Dict, Any, Optional, Union
 
 import numpy as np
 import pandas as pd
 from tabulate import tabulate
 import tensorflow as tf
-import tensorflow_addons as tfa
 
+from sknlp.vocab import Vocab
 from sknlp.data import ClassificationDataset
 from sknlp.metrics import PrecisionWithLogits, RecallWithLogits, FBetaScoreWithLogits
 
@@ -24,10 +24,8 @@ class DeepClassifier(SupervisedNLPModel):
         sequence_length: Optional[int] = None,
         segmenter: Optional[str] = "jieba",
         embedding_size: int = 100,
-        use_batch_normalization: bool = True,
         text2vec: Optional[Text2vec] = None,
-        loss: Optional[str] = None,
-        loss_kwargs: Optional[Dict[str, Any]] = None,
+        loss: str = None,
         **kwargs
     ):
         classes = list(classes)
@@ -46,30 +44,19 @@ class DeepClassifier(SupervisedNLPModel):
             **kwargs
         )
         self._is_multilabel = is_multilabel
-        self._use_batch_normalization = use_batch_normalization
         self._loss = loss
-        self._loss_kwargs = loss_kwargs
 
-    @property
-    def is_multilabel(self):
-        return self._is_multilabel
-
-    @property
-    def use_batch_normalization(self):
-        return self._use_batch_normalization
-
-    def get_loss(self, *args, **kwargs):
-        if self.is_multilabel:
-            if self._loss == "focal":
-                return tfa.losses.SigmoidFocalCrossEntropy(
-                    from_logits=True, **self._loss_kwargs
-                )
+    def get_loss(self):
+        if self._is_multilabel:
             return tf.keras.losses.BinaryCrossentropy(from_logits=True)
         else:
             return tf.keras.losses.CategoricalCrossentropy(from_logits=True)
 
-    def get_metrics(self, *args, **kwargs) -> List[tf.keras.metrics.Metric]:
-        if self.is_multilabel:
+    def get_callbacks(self) -> List[tf.keras.callbacks.Callback]:
+        return []
+
+    def get_metrics(self) -> List[tf.keras.metrics.Metric]:
+        if self._is_multilabel:
             return [
                 PrecisionWithLogits(),
                 RecallWithLogits(),
@@ -82,35 +69,23 @@ class DeepClassifier(SupervisedNLPModel):
                 FBetaScoreWithLogits(self.num_classes, logits2scores="softmax"),
             ]
 
-    @classmethod
-    def get_monitor(cls) -> str:
+    def get_monitor(self) -> str:
         return "val_fbeta_score"
 
     def create_dataset_from_df(
-        self, df: pd.DataFrame, no_label: bool = False
+        self, df: pd.DataFrame, vocab: Vocab, segmenter: str, labels: Sequence[str]
     ) -> ClassificationDataset:
         return ClassificationDataset(
-            self.text2vec.vocab,
-            self.classes,
+            vocab,
+            list(labels),
             df=df,
-            is_multilabel=self.is_multilabel,
-            max_length=self.max_sequence_length,
-            no_label=no_label,
-            text_segmenter=self.text2vec.segmenter,
+            is_multilabel=self._is_multilabel,
+            max_length=self._max_sequence_length,
+            text_segmenter=segmenter,
         )
 
-    def create_dataset_from_csv(
-        self, filename: str, no_label: bool = False
-    ) -> ClassificationDataset:
-        return ClassificationDataset(
-            self.text2vec.vocab,
-            self.classes,
-            csv_file=filename,
-            is_multilabel=self.is_multilabel,
-            max_length=self.max_sequence_length,
-            no_label=no_label,
-            text_segmenter=self.text2vec.segmenter,
-        )
+    def dummy_y(self, X: Sequence[str]) -> List[str]:
+        return ["O" for _ in range(len(X))]
 
     def predict_proba(
         self,
@@ -120,7 +95,7 @@ class DeepClassifier(SupervisedNLPModel):
         batch_size: int = 128
     ) -> np.ndarray:
         logits = super().predict(X=X, dataset=dataset, batch_size=batch_size)
-        return logits2probabilities(logits, self.is_multilabel)
+        return logits2probabilities(logits, self._is_multilabel)
 
     def predict(
         self,
@@ -132,14 +107,14 @@ class DeepClassifier(SupervisedNLPModel):
     ) -> Union[List[str], List[List[str]]]:
         probabilities = self.predict_proba(X=X, dataset=dataset, batch_size=batch_size)
         predictions = probabilities2classes(
-            probabilities, self.is_multilabel, thresholds=thresholds
+            probabilities, self._is_multilabel, thresholds=thresholds
         )
-        if self.is_multilabel:
+        if self._is_multilabel:
             return [
-                [self.idx2class(i) for i in prediction] for prediction in predictions
+                [self._idx2class[i] for i in prediction] for prediction in predictions
             ]
         else:
-            return [self.idx2class(i) for i in predictions]
+            return [self._idx2class[i] for i in predictions]
 
     def score(
         self,
@@ -150,22 +125,18 @@ class DeepClassifier(SupervisedNLPModel):
         thresholds: Union[float, List[float]] = 0.5,
         batch_size: int = 128
     ) -> pd.DataFrame:
-        dataset = self.prepare_dataset(X, y, dataset)
         predictions = self.predict(
-            dataset=dataset, thresholds=thresholds, batch_size=batch_size
+            X=X, dataset=dataset, thresholds=thresholds, batch_size=batch_size
+        )
+        return classification_fscore(
+            y, predictions, self._is_multilabel, classes=list(self._class2idx.keys())
         )
-        return classification_fscore(dataset.y, predictions, classes=self.classes)
 
-    @classmethod
-    def format_score(cls, score_df: pd.DataFrame, format: str = "markdown") -> str:
+    def format_score(self, score_df: pd.DataFrame, format: str = "markdown") -> str:
         return tabulate(score_df, headers="keys", tablefmt="github", showindex=False)
 
     def get_config(self) -> Dict[str, Any]:
-        return {
-            **super().get_config(),
-            "is_multilabel": self.is_multilabel,
-            "use_batch_normalization": self.use_batch_normalization,
-        }
+        return {**super().get_config(), "is_multilabel": self._is_multilabel}
 
     @classmethod
     def _filter_config(cls, config):
@@ -174,11 +145,10 @@ class DeepClassifier(SupervisedNLPModel):
         config.pop("task", None)
         return config
 
-    @classmethod
-    def get_custom_objects(cls) -> Dict[str, Any]:
+    def get_custom_objects(self) -> Dict[str, Any]:
         return {
             **super().get_custom_objects(),
             "PrecisionWithLogits": PrecisionWithLogits,
             "RecallWithLogits": RecallWithLogits,
             "FBetaScoreWithLogits": FBetaScoreWithLogits,
-        }
\ No newline at end of file
+        }
