@@ -1,17 +1,17 @@
-from typing import Sequence, Union, Optional, Dict, Any, List, Tuple, Type
+from typing import Sequence, Union, Optional, Dict, Any, List
 
 import os
 import logging
 
 import tensorflow as tf
+from tensorflow.python.keras.engine import training
 import tensorflow_addons as tfa
-import numpy as np
 import pandas as pd
 
-from sknlp.data import NLPDataset
-from sknlp.data.text_segmenter import get_segmenter
-from sknlp.vocab import Vocab
-from sknlp.callbacks import default_supervised_model_callbacks
+from ..data import NLPDataset
+from ..data.text_segmenter import get_segmenter
+from ..vocab import Vocab
+from ..callbacks import WeightDecayScheduler
 from .text2vec import Word2vec, Text2vec
 from .base_model import BaseNLPModel
 
@@ -25,7 +25,7 @@ class SupervisedNLPModel(BaseNLPModel):
     def __init__(
         self,
         classes: Sequence[str],
-        max_sequence_length: Optional[int] = None,
+        max_sequence_length: int = None,
         sequence_length: Optional[int] = None,
         segmenter: str = "jieba",
         embedding_size: int = 100,
@@ -68,32 +68,12 @@ class SupervisedNLPModel(BaseNLPModel):
     def classes(self) -> List[str]:
         return list(self._class2idx.keys())
 
-    @property
-    def text2vec(self) -> Text2vec:
-        return self._text2vec
-
-    @property
-    def train_dataset(self) -> NLPDataset:
-        return getattr(self, "_train_dataset", None)
-
-    @property
-    def valid_dataset(self) -> NLPDataset:
-        return getattr(self, "_valid_dataset", None)
-
-    def class2idx(self, class_name: str) -> int:
-        return self._class2idx.get(class_name, None)
-
-    def idx2class(self, class_idx: int) -> str:
-        return self._idx2class.get(class_idx, None)
-
     def get_inputs(self) -> tf.Tensor:
-        if getattr(self, "inputs", None) is None:
-            return self.text2vec.get_inputs()
-        else:
-            return self.inputs
+        return self._text2vec.get_inputs()
 
     def get_outputs(self) -> tf.Tensor:
-        return self.build_output_layer(self.build_encode_layer(self.get_inputs()))
+        inputs = self._text2vec.get_outputs()
+        return self.build_output_layer(self.build_encode_layer(inputs))
 
     def build(self) -> None:
         if self._built:
@@ -101,69 +81,60 @@ class SupervisedNLPModel(BaseNLPModel):
         assert self._text2vec is not None
         super().build()
 
+    @classmethod
     def create_dataset_from_df(
-        self, df: pd.DataFrame, vocab: Vocab, segmenter: str, labels: Sequence[str]
+        cls, df: pd.DataFrame, vocab: Vocab, segmenter: str, labels: Sequence[str]
     ) -> NLPDataset:
         raise NotImplementedError()
 
-    def prepare_dataset(
+    def prepare_tf_dataset(
         self,
         X: Sequence[str],
-        y: Union[Sequence[Sequence[str]], Sequence[str], Sequence[float]],
+        y: Sequence[str],
         dataset: NLPDataset,
-    ) -> NLPDataset:
-        assert (X is not None and y is not None) or dataset is not None
-        if dataset is not None:
-            return dataset
-        if isinstance(y[0], (list, tuple)) and isinstance(y[0][0], str):
-            y = ["|".join(map(str, y_i)) for y_i in y]
-        if isinstance(X[0], (list, tuple)):
-            df = pd.DataFrame(zip(*X, y))
-        else:
-            df = pd.DataFrame(zip(X, y))
-        return self.create_dataset_from_df(
-            df, self.text2vec.vocab, self.text2vec.segmenter, self.classes
-        )
+        batch_size: int,
+        shuffle: bool = True,
+    ) -> tf.data.Dataset:
+        assert not ((X is None or y is None) and dataset is None)
 
-    def compile_optimizer(self, optimizer_name, **kwargs) -> None:
-        if not self._built:
-            self.build()
-        weight_decay = kwargs.pop("weight_decay", 0)
-        if optimizer_name != "adam":
-            optimizer = tf.keras.optimizers.deserialize(
-                {"class_name": optimizer_name, "config": kwargs}
-            )
-        elif weight_decay > 0:
-            optimizer = tfa.optimizers.AdamW(weight_decay, **kwargs)
-        else:
-            optimizer = tfa.optimizers.LazyAdam(**kwargs)
-        self._model.compile(
-            optimizer=optimizer,
-            loss=self.get_loss(),
-            metrics=self.get_metrics(),
+        if self._text2vec is None:
+            cut = get_segmenter(self._segmenter)
+            vocab = self.build_vocab(X, cut)
+            self._text2vec = Word2vec(vocab, self._embedding_size, self._segmenter)
+
+        if dataset is not None:
+            return dataset.batchify(batch_size, shuffle=shuffle)
+
+        if isinstance(y[0], (list, tuple)):
+            y = ["|".join(map(str, y_i)) for y_i in y]
+        df = pd.DataFrame(zip(X, y), columns=["text", "label"])
+        dataset = self.create_dataset_from_df(
+            df, self._text2vec.vocab, self._text2vec.segmenter, self.classes
+        )
+        return dataset.batchify(
+            batch_size, shuffle=shuffle, shuffle_buffer_size=df.shape[0]
         )
 
     def fit(
         self,
         X: Sequence[str] = None,
-        y: Union[Sequence[Sequence[str]], Sequence[str], Sequence[float]] = None,
+        y: Union[Sequence[Sequence[str]], Sequence[str]] = None,
         *,
         dataset: NLPDataset = None,
         valid_X: Sequence[str] = None,
-        valid_y: Union[Sequence[Sequence[str]], Sequence[str], Sequence[float]] = None,
+        valid_y: Union[Sequence[Sequence[str]], Sequence[str]] = None,
         valid_dataset: NLPDataset = None,
         batch_size: int = 128,
-        n_epochs: int = 10,
+        n_epochs: int = 30,
         optimizer: str = "adam",
-        optimizer_kwargs: Optional[Dict[str, Any]] = None,
-        learning_rate: float = 1e-3,
-        weight_decay: float = 0.0,
-        clip: Optional[float] = 5.0,
-        learning_rate_update_factor: float = 0.5,
-        learning_rate_update_epochs: int = 10,
+        lr: float = 1e-3,
+        weight_decay: float = 1e-4,
+        lr_update_factor: float = 0.5,
+        lr_update_epochs: int = 10,
+        clip: float = 5.0,
         enable_early_stopping: bool = False,
         early_stopping_patience: Optional[int] = None,
-        early_stopping_min_delta: float = 0.0,
+        early_stopping_min_delta: float = 0,
         early_stopping_use_best_epoch: bool = False,
         early_stopping_monitor: int = 2,  # 1 for loss, 2 for metric
         checkpoint: Optional[str] = None,
@@ -171,54 +142,69 @@ class SupervisedNLPModel(BaseNLPModel):
         log_file: Optional[str] = None,
         verbose: int = 2
     ) -> None:
-        if self._text2vec is None:
-            cut = get_segmenter(self.segmenter)
-            vocab = self.build_vocab(X or dataset.text, cut)
-            self._text2vec = Word2vec(vocab, self.embedding_size, self.segmenter)
+        assert not (
+            self._text2vec is None and X is None
+        ), "When text2vec is not given, X must be provided"
 
-        self._train_dataset = self.prepare_dataset(X, y, dataset)
-        if (valid_X is None or valid_y is None) and valid_dataset is None:
-            self._valid_dataset = None
-        else:
-            self._valid_dataset = self.prepare_dataset(valid_X, valid_y, valid_dataset)
-        train_tf_dataset = self.train_dataset.batchify(batch_size)
-        valid_tf_dataset = None
-        if self.valid_dataset is not None:
-            valid_tf_dataset = self.valid_dataset.batchify(batch_size, shuffle=False)
-
-        optimizer_kwargs = optimizer_kwargs or dict()
-        optimizer_kwargs = {
-            "learning_rate": learning_rate,
-            "weight_decay": weight_decay,
-            **optimizer_kwargs,
-        }
-        if clip is not None:
-            optimizer_kwargs["clipnorm"] = clip
-        self.compile_optimizer(optimizer, **optimizer_kwargs)
-
-        monitor = "val_loss"
-        monitor_direction = "min"
-        if early_stopping_monitor == 2 and self.get_monitor():
-            monitor = self.get_monitor()
-            monitor_direction = "max"
-        callbacks = default_supervised_model_callbacks(
-            learning_rate_update_factor=learning_rate_update_factor,
-            learning_rate_update_epochs=learning_rate_update_epochs,
-            use_weight_decay=weight_decay > 0,
-            enable_early_stopping=enable_early_stopping,
-            early_stopping_monitor=monitor,
-            early_stopping_monitor_direction=monitor_direction,
-            early_stopping_patience=early_stopping_patience,
-            early_stopping_min_delta=early_stopping_min_delta,
-            early_stopping_use_best_epoch=early_stopping_use_best_epoch,
-            log_file=log_file,
+        self.train_dataset = self.prepare_tf_dataset(
+            X,
+            y,
+            dataset,
+            batch_size,
         )
-        callbacks.extend(self.get_callbacks(batch_size))
+        if (valid_X is None or valid_y is None) and valid_dataset is None:
+            self.valid_dataset = None
+        else:
+            self.valid_dataset = self.prepare_tf_dataset(
+                valid_X,
+                valid_y,
+                valid_dataset,
+                batch_size,
+                shuffle=False,
+            )
+        if not self._built:
+            self.build()
+        optimizer = tfa.optimizers.AdamW(weight_decay, learning_rate=lr, clipnorm=clip)
+        if optimizer != "adam":
+            pass
+        loss = self.get_loss()
+        metrics = self.get_metrics()
+        callbacks = self.get_callbacks()
+        self._model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
 
+        def lr_scheduler(epoch, lr):
+            return lr * lr_update_factor ** ((epoch + 1) % lr_update_epochs == 0)
+
+        def wd_scheduler(epoch, wd):
+            return wd * lr_update_factor ** ((epoch + 1) % lr_update_epochs == 0)
+
+        lr_decay = tf.keras.callbacks.LearningRateScheduler(
+            lr_scheduler, verbose=verbose
+        )
+        weight_decay = WeightDecayScheduler(wd_scheduler, verbose=verbose)
+        callbacks = [*callbacks, lr_decay, weight_decay]
+        if enable_early_stopping:
+            if early_stopping_monitor == 2 and self.get_monitor():
+                mode = "max"
+                monitor = self.get_monitor()
+            else:
+                mode = "min"
+                monitor = "val_loss"
+            callbacks.append(
+                tf.keras.callbacks.EarlyStopping(
+                    monitor=monitor,
+                    min_delta=early_stopping_min_delta,
+                    patience=early_stopping_patience or lr_update_epochs,
+                    mode=mode,
+                    restore_best_weights=early_stopping_use_best_epoch,
+                )
+            )
+        if log_file is not None:
+            callbacks.append(tf.keras.callbacks.CSVLogger(log_file))
         self._model.fit(
-            train_tf_dataset,
+            self.train_dataset,
             epochs=n_epochs,
-            validation_data=valid_tf_dataset if self.valid_dataset else None,
+            validation_data=self.valid_dataset,
             callbacks=callbacks,
             verbose=verbose,
         )
@@ -232,17 +218,18 @@ class SupervisedNLPModel(BaseNLPModel):
         *,
         dataset: NLPDataset = None,
         batch_size: int = 128
-    ) -> np.ndarray:
+    ) -> tf.Tensor:
         assert self._built
         y = None if X is None else self.dummy_y(X)
-        dataset = self.prepare_dataset(X, y, dataset)
-        return self._model.predict(dataset.batchify(batch_size, shuffle=False))
+        dataset = self.prepare_tf_dataset(X, y, dataset, batch_size, shuffle=False)
+        return self._model.predict(dataset)
 
     def score(
         self,
         X: Sequence[str] = None,
-        y: Union[Sequence[Sequence[str]], Sequence[str], Sequence[float]] = None,
+        y: Union[Sequence[Sequence[str]], Sequence[str]] = None,
         *,
+        dataset: NLPDataset = None,
         thresholds: Union[float, Dict[str, float]] = 0.5,
         batch_size: int = 128
     ) -> pd.DataFrame:
@@ -258,7 +245,7 @@ class SupervisedNLPModel(BaseNLPModel):
 
     def save(self, directory: str) -> None:
         super().save(directory)
-        self.text2vec.save_vocab(directory)
+        self._text2vec.save_vocab(directory)
 
     @classmethod
     def load(cls, directory: str) -> "SupervisedNLPModel":
@@ -276,8 +263,7 @@ class SupervisedNLPModel(BaseNLPModel):
     def export(self, directory: str, name: str, version: str = "0") -> None:
         super().export(directory, name, version)
         d = os.path.join(directory, name, version)
-        self.text2vec.save_vocab(d)
+        self._text2vec.save_vocab(d)
 
-    @classmethod
-    def get_custom_objects(cls) -> Dict[str, Any]:
+    def get_custom_objects(self) -> Dict[str, Any]:
         return {**super().get_custom_objects(), "AdamW": tfa.optimizers.AdamW}
